#!/usr/bin/python2
# coding: utf-8
# author: Florian Preinstorfer <nblock@archlinux.us>

import logging
import sys

from proteus import Model
from proteus import config


# Configuration
TRYTON_HOST="The tryton host"
TRYTON_PORT="8000"
TRYTON_DATABASE="The name of the database"
TRYTON_USER = "The tryton user"
TRYTON_PASSWORD = "The tryton password"
TRYTON_VERBOSITY="DEBUG"
# The ID of the category which should be user for accounting
TRYTON_NEW_ACCOUNTING_CATEGORY_ID = 74


def configure_connection(host, port, username, password, database):
    logging.debug(
        'Configure connection: host={}, port={}, username={}, password='
        '<snipped>, database={}'.format(host, port, username, database))
    config.set_xmlrpc('https://{}:{}@{}:{}/{}/'.format(
        username, password, host, port, database))


def configure_logging(verbosity):
    logging.basicConfig(level=getattr(logging, verbosity))
    logging.debug('Logging configured')


def migrate_account_category(account_category_id):
    # Resolve destination account category.
    try:
        account_category = Model.get('product.category').find(
            [['id', '=', account_category_id]])[0]
    except IndexError as ex:
        logging.error('Could not find the requested accounting category.')
        logging.exception(ex)
        return

    # Process all product templates.
    for record in Model.get('product.template').find():
        logging.debug(u'Processing product: {}'.format(record.name))

        # Check for a complete migration.
        if record.account_category == account_category and record.categories:
            logging.warning(u'The migration seems to be complete – Skipping.')
            continue

        # Copy the migrated account category as regular category.
        if record.account_category and record.account_category not in record.categories:
            record.categories.append(record.account_category)

        # Apply new account category.
        if record.account_category != account_category:
            record.account_category = account_category

        # Save
        record.save()


def migrate_category_settings(account_category_id):
    # Remove the accounting information from all categories expect from the one
    # with account_category_id.
    while True:
        # Proceed until all accounting flags got cleared.
        records = Model.get('product.category').find(
            [["id", "!=", account_category_id], ["accounting", "=", True]])
        if not records:
            logging.debug(u'Category migration complete.')
            break

        # Process each category
        for record in records:
            logging.debug(u'Processing category: {}'.format(record.name))

            #  Remove accounting
            record.account_parent = False
            record.account_revenue = None
            record.account_expense = None

            # Remove taxing
            record.taxes_parent = False

            # Remove accounting flag.
            record.accounting = False

            # Save
            record.save()


def main():
    configure_logging(TRYTON_VERBOSITY)
    configure_connection(TRYTON_HOST, TRYTON_PORT, TRYTON_USER, TRYTON_PASSWORD, TRYTON_DATABASE)
    migrate_account_category(TRYTON_NEW_ACCOUNTING_CATEGORY_ID)
    migrate_category_settings(TRYTON_NEW_ACCOUNTING_CATEGORY_ID)


if __name__ == '__main__':
    main()

# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4 smartindent autoindent
